/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/**
 * Token.java
 * It results: 
 * code of  open tags > 0, close tags < 0 and simmetric
 * code of pcdata = 0,
 * code(<X>)=-code(</X>)
 * Created: Sat Jul  6 18:52:41 2002
 *
 * @author  Valter Crescenzi
 * @version
 */

package roadrunner.ast;

import roadrunner.parser.Token;

public abstract class ASTToken extends NodeAdapter implements Token, Matchable, Node {
    
    protected int depth;
    protected int code;
    
    /* Token with code == 0 is associated with all pcdata */
    /** Token with depth implementation to be reused, not instantiated */
    public ASTToken(int depth, int code) {
        super(ASTConstants.JJTTOKEN);
        this.depth = depth;
        this.code = code;
    }
    
    /** Accept the visitor. **/
    public boolean jjtAccept(Visitor visitor) {
        return visitor.visit((ASTToken)this);
    }
    
    public Node clone(boolean deep) {
        //Flyweight pattern: these nodes are immutable
        return this; // these kind of leaves are shared
    }
    
    
    public int code() { return this.code; }
    
    public int depth() { return this.depth; }
    
    public boolean isPCDATA()  { return (this.code==0); }
    public boolean isTag()     { return (this.code!=0); }
    public boolean isStartTag() { return (this.code>0); }
    public boolean isEndTag()   { return (this.code<0); }
    
    
  /* two PCDATA match iff they have the same depth
     two TAGs match iff they have same element, atts, and depth */
    public boolean matches(Token that) {
        return this.code() == that.code() && this.depth() == that.depth();
    }
    
    public boolean maybeVariant() {
        // only start tags as possible variants
        return (this.isPCDATA()) || (this.isStartTag() && (this.isIMG() || this.isLink()));
    }
    
    public boolean equals(Object o) {
        if (this.hashCode()!=o.hashCode()) return false;
        if (!super.equals(o)) return false;
        ASTToken that  = (ASTToken)o;
        return this.code == that.code && this.depth == that.depth;
    }
    
    public int hashCode() {
        return 1000*depth+code;
    }
    
}
