/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * Region.java
 *
 *An abstract class to save regions of structures such as AST or TokenList.
 * A region object does not depend on the specific AST or TokenList instance 
 * object and can be used to mark regions over clones of the same structure.
 *
 * Created on 25 gennaio 2003, 12.01
 * @author  Valter Crescenzi
 */
package roadrunner.bidi;

import java.util.List;
import java.util.Iterator;

public abstract class Region {

    private int lborder;    // A couple of integers which delimit the region itself.
    private int rborder;
    
    protected Region(int left, int right) {
        this.lborder = left;
        this.rborder = right;
    }
    
    public Object getFirst() {
        return asList().get(0);
    }

    public Object getLast() {
        return asList().get(size()-1);
    }
    
    public int getLeftBorderPos() {
        return this.lborder;
    }
    
    public int getRightBorderPos() {
        return this.rborder;
    }
    
    public int size() {
        return this.rborder - this.lborder;
    }
    
    public boolean isEmpty() {
        return this.size()==0;
    }
    
    abstract public List asList();
    
    abstract public Region subRegion(int l, int r);
    
    protected static void checkBorders(int size, int left, int right) {
        if (left<0 || right>size || left>right)
            throw new IndexOutOfBoundsException("Bad region delimiters: ("+left+","+right+")"+
            " out of "+size+" elements");
    }
    
    public String toString() {
        return toString(getLeftBorderPos());
    }
    
    public String toString(int fromIndex) {
        StringBuffer result = new StringBuffer(size()*10);
        int index = fromIndex;
        Iterator it = asList().iterator();
        if (it.hasNext()) result.append(index+"."+it.next());
        while (it.hasNext()) {
            index++;
            result.append(", "+index+"."+it.next());
        }
        return result.toString();
    }
    
}
