/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * ExpressionIterator.java
 *
 * Created on 27 marzo 2003, 18.04
 */

package roadrunner.engine;

/**
 *
 * @author  Valter Crescenzi
 */

import roadrunner.ast.Expression;
import java.util.NoSuchElementException;

public abstract class ExpressionIterator {
    
    private static int counter = 0;
    
    // Si potrebbe passare ad una interfaccia + due implementazioni di supporto:
    // quella attuale pi SingletonExpressionIterator che servirebbe
    // per il dummy state iniziale di SearchSpace e per gli addHook.
    // No: troppo articolato per usarlo dentro il dummy state e benefici trascurabili.
    // Pattern Iterator + Template Method
    
    private Expression nextResult; // cache of next result
    
    protected int id;
    
    protected ExpressionIterator() {
        this.nextResult = null;
        this.id = counter++;
    }
    
    public int getId() {
        return this.id;
    }
    
    /* This method has to compute next result if exists, null otherwise */
    abstract protected Expression computeNext();
    
    public boolean hasNext() {
        if (this.nextResult!=null) return true; //check if already computed
        this.nextResult = computeNext();
        return (this.nextResult!=null);
    }
    
    public Expression next() {
        if (this.hasNext()) {
            Expression result = this.nextResult;
            this.nextResult = null;
            return result;
        }
        throw new NoSuchElementException(this+" has already expired!");
    }
    
    protected static ExpressionIterator iteratorOverSingleExp(final Expression single) {
        // create a dummy ExpressionIterator for initial expression
        return new ExpressionIterator() {
            private boolean expired = false;
            protected Expression computeNext() {
                if (this.expired) return null;
                this.expired = true; // just one result;
                return single;
            }
        };
    }
    
    protected static ExpressionIterator expiredIterator() {
        return new ExpressionIterator() {
            protected Expression computeNext() {
                return null;
            }
        };
    }
}
