/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * LLChecker.java
 *
 * Created on 4 ottobre 2003, 10.23
 * @author  Valter Crescenzi
 */

package roadrunner.engine;

import java.util.List;
import java.util.Iterator;
import java.util.logging.*;

import roadrunner.bidi.Region;
import roadrunner.bidi.Direction;
import roadrunner.ast.Visitor;
import roadrunner.ast.PrefixHunter;
import roadrunner.parser.Token;


public class LLChecker extends PrefixHunter implements Visitor {
    
    static private Logger log = Logger.getLogger(LLChecker.class.getName());
    
    /** Creates a new instance of LLChecker */    
    public LLChecker(int k, Direction dir) {
        super(k,dir);
    }           
    
    public  boolean existsAcommonPrefix(Region r1, Region r2) {
        if (getK()==-1) return false;
        return existsAcommonPrefix(getPrefixes(r1), getPrefixes(r2));
    }
    
    private boolean existsAcommonPrefix(List prefixes1, List prefixes2) {
        Iterator it1 = prefixes1.iterator();
        while (it1.hasNext()) {
            List prefix1 = (List)it1.next();
            Iterator it2 = prefixes2.iterator();
            while (it2.hasNext()) {
                List prefix2 = (List)it2.next();
                if (matches(prefix1, prefix2)) {
                   return true;
                }
            }
        }
        return false;
    }
    
    private boolean matches(List prefix1, List prefix2) {
        Iterator it1 = prefix1.iterator();
        Iterator it2 = prefix2.iterator();
        while (it1.hasNext()) {
            Token token1 = (Token)it1.next();
            if (token1 == null) return true; //null stands for subtree
            if (it2.hasNext()) {
                Token token2 = (Token)it2.next();
                if (token2 == null) return true;
                if (!token1.matches(token2)) return false;
            }
            else return true;
        }
        return true;
    }
    
}
