/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
package roadrunner.engine.space;

/**
 * Operator.java
 *
 *
 * Created: Thu Dec 21 09:57:49 2000
 *
 * @author Valter Crescenzi
 * @version
 */

import roadrunner.ast.Expression;

public interface Operator extends Comparable {

  /** Questa interfaccia deve supportare operatori che restituiscono soluzioni multiple;
   *  in particolare, mentre con gli addHook  possibile prevedere il numero di soluzioni
   *  che possono essere prodotte contando il numero di occorrenze del delimitatore. Nel
   *  Caso dell'addPlus  impossibile. Nel primo caso si possono costruire oggetti diversi
   *  per ciascuna soluzione e rientrare nel modello classico, nel secondo caso no.
   **/
    
  public int getId();
  
  public Expression next();
  
  public boolean hasNext();
  
  public int h();
  
  public int compareTo(Object o);

  /** Sull'ordine degli operatori: permettere agli operatori con square multiplo di precedere
   * quelli a square semplice comporta che vengono velocemente prodotte exp reg in cui i
   * pattern ripetuti sono collassati a due a due, oppure a tre a tre e cos via.
   * Poi un inevitabile mismatch risolve
   * tutto inserendo un plus di plus. Questo pu introdurre "mismatch ricorsivi" che allo
   * stato attuale addPlus_w non sa come gestire. Varie possibilit:
   * - eliminare i mismatch di bordo alla radice: cio direttamente in Parser
   * - rifiutare gli addPlus che introducono i doppi plus
   * - [(a regime) ordinare gli square di modo che i corti vengano provati prima
   */
  
} // Operator
