/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/**
 * State.java
 *
 *
 * Created: Tue Dec 28 12:57:51 1999
 *
 * @author Valter Crescenzi
 * @version
 */

package roadrunner.engine.space;

import java.util.*;
import java.util.logging.*;

import roadrunner.ast.Expression;
import roadrunner.engine.ExpressionIterator;

class State extends ExpressionIterator  {
    
    static private Logger log = Logger.getLogger(State.class.getName());
    
    /* This class contains all information to recover the state of the matching process */

    private SortedSet operators;
    private SearchSpace space;
    
    State(List mismatches, SearchSpace space) {
        this.space = space;
        log.finest("Space "+space.getId()+" - Creating operators for State "+getId());
        this.operators = space.getOperatorFactory().createOperators(mismatches);
        log.fine("Space "+space.getId()+" - State "+getId()+": there are "+operators.size()+" operator(s) to apply\n"+operators);
    }

    protected Expression computeNext() {
        while (!this.operators.isEmpty()) {
            Operator op = (Operator)(this.operators.first());
            log.finest("Space "+space.getId()+" - Evaluating operator "+op.getId()+" from State "+getId()+":\n"+op);
            if (op.hasNext()) {
                log.fine("Space "+space.getId()+" - Applying operator "+op.getId()+" from State "+getId()+":\n"+op);
                this.operators.add(op); // N.B. its order could change
                return op.next();
            }
            else {
                log.finest("Space "+space.getId()+" - Operator "+op.getId()+" from State "+getId()+" has expired");
                this.operators.remove(op);
            }
        }
        return null;
    }
    
} // State
