/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * Combiner.java
 *
 * Created on 7 dicembre 2003, 16.01
 * @author  Valter Crescenzi, Luigi Arlotta
 */

package roadrunner.labeller;

import java.util.*;
import java.util.logging.*;

import roadrunner.ast.ASTVariant;
import roadrunner.parser.Token;

/**
 *
 * @author  Luigi Arlotta, Valter Crescenzi
 */
class Combiner {
    
    private static Logger log = Logger.getLogger(Combiner.class.getName());
    
    private List variants;
    private List assocs;
    private IdentityHashMap var2inv;
    
    /** Creates a new instance of Combiner */
    Combiner(List variants,List assocs) {
        this.variants = variants;
        this.assocs = assocs;
        this.var2inv = new IdentityHashMap();
        init();
    }
    
    private void init() {
        Iterator varIt = this.variants.iterator();
        while (varIt.hasNext()) {
            ASTVariant variant = (ASTVariant)varIt.next();
            Token invariant = null;
            String label = null;
            Iterator assocIt = this.assocs.iterator();
            while (assocIt.hasNext()) {
                IdentityHashMap var2inv = (IdentityHashMap)assocIt.next();
                invariant = (Token)var2inv.get(variant);
                //discard white-space invariants
                if (invariant!=null) {
                    if (label==null) label = invariant.getText();
                    if (!label.equals(invariant.getText())) {
                        log.fine("Discarding annotation of "+variant+" since it is associated "+
                                 "both to \""+label+"\" and \""+invariant.getText()+"\"");
                        label = null;
                        break;
                    }
                }
            }
            if (label!=null) {
                log.fine("Combined annotation of "+variant+" is \""+invariant+"\"");
                this.var2inv.put(variant, invariant);
            }
        }
        log.fine("Final annotations: "+this.var2inv);
    }
        
    public IdentityHashMap combine() {
        return this.var2inv;
    }
    
    
}
