/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * TextExtractor.java
 *
 * Created on 28 gennaio 2004, 14.48
 */

package roadrunner.labeller;

import roadrunner.util.Util;
import roadrunner.config.Config;
import roadrunner.config.Constants;

public class LabelExtractor implements Constants {
    
    private boolean trimming;
    private int minLabelLength;
    private int maxLabelLength;
    
    /** Creates a new instance of TextExtractor */
    LabelExtractor() {
        this.trimming       = Config.getPrefs().getBoolean(TRIM_LABELS);
        this.maxLabelLength = Config.getPrefs().getInt(MAX_LABEL_LEN);
        this.minLabelLength = Config.getPrefs().getInt(MIN_LABEL_LEN);
    }
    
    private boolean isLabelTrimmingEnabled() {
        return this.trimming;
    }
    
    public String extractLabel(String s) {
        if (s==null || Util.isAllWhiteSpaceChars(s)) return null;
        String label = forceLabelLength(isLabelTrimmingEnabled() ? trimLabel(s) : s);
        return label;
    }
    
    static public String trimLabel(String s) {
        // Given a string extract a label by trimming punctuation chars
        StringBuffer result = new StringBuffer(s.length());
        //find first feasible char
        int start = -1, end = s.length();
        for (int i=0; i<s.length(); i++) {
            if (isLabelStart(s.charAt(i))) {
                start = i;
                break;
            }
        }
        if (start== -1) return ""; //no label to extract
        end = s.length()-1;
        for (int j = s.length()-1; j > start; j--) {
            if (isLabelEnd(s.charAt(j))) {
                end = j+1;
                break;
            }
        }
        return s.substring(start, end).trim();
    }
    
    private String forceLabelLength(String label) {
        // discard labels too short and truncate labels too long
        if (label.length()<minLabelLength) return null;
        if (label.length()>maxLabelLength) return null;
        return label;
    }
    
    static private boolean isLabelStart(char ch) {
        return Character.isUnicodeIdentifierStart(ch);
    }
    
     static private boolean isLabelEnd(char ch) {
        return Character.isUnicodeIdentifierPart(ch);
    }
}
