/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * ParserListenerAdapter.java
 *
 * @author  Valter Crescenzi
 * Created on 24 aprile 2003, 20.51
 */

package roadrunner.parser;

import roadrunner.ast.*;

public class ParserListenerAdapter implements ParserListener {
  
  public void startAnd(ASTAnd and)      { startNode(and);  }
  public void startHook(ASTHook hook)   { startNode(hook); }
  public void startPlus(ASTPlus plus)   { startNode(plus); }
  public void startSubtree(ASTSubtree s) { startNode(s);    }
  public void startVariant(ASTVariant v) { startNode(v);    }
  public void startToken(ASTToken node)  { startNode((Node)node); }
  
  public void endAnd(ASTAnd and, boolean matches)                 { endNode(and, matches); }
  public void endHook(ASTHook hook, boolean matches, int times)   { endNode(hook, matches); }  
  public void endPlus(ASTPlus plus, boolean matches, int times)   { endNode(plus, matches); }  
  public void endSubtree(ASTSubtree s, boolean matches, int f, int l) { endNode(s, matches); }  
  public void endVariant(ASTVariant v, boolean matches, Token t)  { endNode(v, matches);     }
  public void endToken(ASTToken node, boolean matches, Token t) { endNode((Node)node, matches); }
  
  public void startNode(Node node)                {}
  public void endNode(Node node, boolean matches) {}  
  
}
