/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * Region.java
 *
 * Created on 25 gennaio 2003, 12.01
 * @author  Valter Crescenzi
 */

package roadrunner.parser;

import java.util.List;
import roadrunner.bidi.Region;

public class TokenListRegion extends Region {
  
  private TokenList tokenlist;
  
  /** Creates a new instance of Region for TokenList */
  public TokenListRegion(TokenList tokenlist, int left, int right) {
    super(left,right);
    this.tokenlist = tokenlist;
    checkBorders(tokenlist.size(),left,right);
  }
  
  /** Creates a new instance of Region for all tokens of a TokenList */
  public TokenListRegion(TokenList tokenlist) {
    this(tokenlist, 0, tokenlist.size());
  }
  
  public TokenList getTokenlist() { return this.tokenlist; }
  
  public Region subRegion(int l, int r) {
    return new TokenListRegion(getTokenlist(), l, r);
  }
  
  public List asList() {
    return getTokenlist().getTokens().subList(getLeftBorderPos(),getRightBorderPos());
  }
  
  
}
