/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * LLCheckerTest.java
 * JUnit based test
 *
 * Created on 8 ottobre 2003, 14.29
 * @author Valter Crescenzi
 */

package roadrunner.engine;

import junit.framework.*;

import java.util.*;
import java.util.logging.*;

import roadrunner.bidi.*;
import roadrunner.ast.*;
import roadrunner.parser.Token;

public class LLCheckerTest extends TestCase {
    
    public LLCheckerTest(java.lang.String testName) {
        super(testName);
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite(LLCheckerTest.class);
        return suite;
    }
    
    
    //         And
    //   /    |    \   \
    //  A    Hook   C   Variant
    //        \
    //        And
    //         \
    //          B
    
    /** Test of getPrefixes method, of class roadrunner.engine.LLChecker. */
    public void testGetPrefixesWithHook() {
        System.out.println("testGetPrefixesWithHook");
        Expression exp = ASTBuilderTest.getFixExpression();
        Direction dir = Direction.LEFT2RIGHT;
        LLChecker checker = new LLChecker(3,dir);
        List prefixes = checker.getPrefixes(exp.asRegion());
        assertEquals(2, prefixes.size());
        
        System.out.println(dir+" "+prefixes);
        
        dir = Direction.RIGHT2LEFT;
        checker = new LLChecker(3,dir);
        prefixes = checker.getPrefixes(exp.asRegion());
        assertEquals(2, prefixes.size());
        
        System.out.println(dir+" "+prefixes);
    }
    
    /** Test of getPrefixes method, of class roadrunner.engine.LLChecker. */
    public void testGetPrefixesWithPlus() {
        System.out.println("testGetPrefixesWithPlus");
        //      And
        //   /    |    \         \
        //  A    Plus  C  Variant
        //        \
        //        And
        //         \
        //          B
        Expression exp = ASTBuilderTest.getFixExpressionWithPlus();
        Direction dir = Direction.LEFT2RIGHT;
        LLChecker checker = new LLChecker(3,dir);
        int [] indicesOfB = {1,0,0};
        Node.Path pathOfB = new PathArray(indicesOfB);
        List prefixes = checker.getPrefixes(exp.asRegion());
        System.out.println(dir+" "+prefixes);
        assertEquals(2,prefixes.size());
        List prefix = (List)prefixes.get(0);
        assertEquals(3,prefix.size());
        assertEquals(exp.getNode(pathOfB), prefix.get(1));
        prefix = (List)prefixes.get(1);
        assertEquals(exp.getNode(pathOfB), prefix.get(1));
        
        checker = new LLChecker(2,dir);
        prefixes = checker.getPrefixes(exp.asRegion());
        assertEquals(1,prefixes.size());
    }
    
    
    // Add test methods here, they have to start with 'test' name.
    // for example:
    // public void testHello() {}
    
    
}
