/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * TestOfTheYearTest.java
 * JUnit based test
 *
 * Created on 16 ottobre 2003, 17.54
 * @author Valter Crescenzi
 */

package roadrunner.engine;

import junit.framework.*;

import java.io.*;
import java.util.*;

import roadrunner.ast.Expression;
import roadrunner.parser.TokenList;
import roadrunner.Wrapper;
import roadrunner.engine.sampler.*;

import roadrunner.FixtureOnFile;

public class TestOfTheYearTest extends TestCase {
    
    private static String  wfilename   = "rufusWrapper2.xml";
    
    public TestOfTheYearTest(java.lang.String testName) {
        super(testName);
    }
    
    public static void main(java.lang.String[] args) {
        junit.textui.TestRunner.run(suite());
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite(TestOfTheYearTest.class);
        return suite;
    }
    
    // Add test methods here, they have to start with 'test' name.
    // for example:
    // public void testHello() {}
    
    public void testOfTheYear() throws Exception {
        Wrapper wrapper = Wrapper.load(FixtureOnFile.getWrapperFile(wfilename));
        CharacteristicSet samples = new CharacteristicSet(wrapper.getExpression());
        Iterator it = samples.listIterator();
        List chi = new LinkedList();
        while (it.hasNext()) {
            chi.add(it.next());
        }
        TokenList[] tokenlists = (TokenList[])chi.toArray(new TokenList[0]);
        Engine engine = new Engine(wrapper.getExpression(), tokenlists,0);
        Set solutions = engine.match();
        assertTrue(solutions.size()>0);
        // Assert that original expression exists in the set of solutions
        assertTrue(solutions.contains(wrapper.getExpression()));
        
        // Assert that it is the FIRST solution!        
        Expression solution = (Expression)solutions.iterator().next();        
        assertEquals(wrapper.getExpression(), solution);
        System.out.println(solution.dump(""));
    }
    
    
}
