/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License
*/
/*
 * FreeTextHunterTest.java
 * JUnit based test
 *
 * Created on 28 febbraio 2003, 17.38
 * @author Valter Crescenzi
 */

package roadrunner.parser;

import junit.framework.*;

import java.io.*;
import java.util.*;
import java.util.logging.*;

import org.w3c.dom.*;
import org.xml.sax.SAXException;

import roadrunner.config.*;
import roadrunner.labeller.Box;
import roadrunner.util.DOMLoader;

import roadrunner.FixtureOnFile;

public class FreeTextHunterTest extends TestCase {
    
    private final static Logger log = Logger.getLogger(FreeTextHunterTest.class.getName());
    
    private static String sample_fn  = "freetext.html";
    private static String prefs_fn   = "freepref.xml";
    
    private Document freedoc;
    private Preferences freeprefs;
    
    public void setUp() throws Exception {
        FileReader reader = new FileReader(FixtureOnFile.getSampleFile(sample_fn));
        this.freedoc = DOMLoader.parseHTML(reader);
        this.freeprefs = new Preferences();
        this.freeprefs.load(FixtureOnFile.getPreferencesFile(prefs_fn));
    }
    
    
    public FreeTextHunterTest(java.lang.String testName) {
        super(testName);
    }
    
    public static void main(java.lang.String[] args) {
        junit.textui.TestRunner.run(suite());
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite(FreeTextHunterTest.class);
        
        return suite;
    }
    
    /** Test of getCollapsedDocument method, of class roadrunner.parser.FreeTextHunter. */
    public void testGetCollapsedDocument() throws Exception {
        System.out.println("testGetCollapsedDocument");
        
        FreeTextHunter hunter = new FreeTextHunter(this.freedoc,this.freeprefs);
        Document collapsed = hunter.getCollapsedDocument();
        assertNotNull(collapsed);
        log.info(collapsed.getDocumentElement().toString());
        Element strong = (Element)collapsed.getElementsByTagName("STRONG").item(0);
        assertNotNull(strong);
        String content = strong.getChildNodes().item(1).getNodeValue();
        // 'Yah!' and 'collapsed' must end up in the same DOM text node
        assertTrue(content.indexOf("Yah!")!=-1 && content.indexOf("collapsed")!=-1);
    }
    
    public void testFreeTextAndBoundingBoxes() throws Exception {
        System.out.println("testFreeTextAndBoundingBoxes");
        
        FileReader reader = new FileReader(FixtureOnFile.getSampleFile(sample_fn));
        Lexer lexer = new Lexer(reader, this.freeprefs);
        List tokens = lexer.getTokens();
        Map token2bb = lexer.getTokens2BoundingBoxes();
        Token yah = findTextTokenContaining(tokens, "Yah!");
        assertNotNull("The expected text token containing \'Yah!\' has not been detected!", yah);
        Box yahbox = (Box)token2bb.get(yah);
        assertEquals(new Box(10,10, 90,30), yahbox);
        Token x = findTextTokenContaining(tokens, "X");
        assertNotNull("The expected text token containing \'X\' has not been detected!", x);
        Box xbox = (Box)token2bb.get(x);
        assertEquals(new Box(0,0, 20,20), xbox);
        Token zxy = findTextTokenContaining(tokens, "ZXY");
        assertNotNull("The expected text token containing \'ZXY\' has not been detected!", zxy);
        Box zxybox = (Box)token2bb.get(zxy);
        assertEquals(new Box(20,20, 30,30), zxybox);
    }
    
    private Token findTextTokenContaining(List tokens, String wanted) {
        Iterator it = tokens.iterator();
        while (it.hasNext()) {
            Token token = (Token)it.next();
            if (token.isPCDATA() && token.getText().indexOf(wanted)!=-1) {
                return token;
            }
        }
        return null;
    }
    
    
}
